@@ -7,8 +7,8 @@
  * netdump dumpfiles, the facilities in netdump.c are used.  For
  * compressed dumpfiles, the facilities in this file are used.
  *
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 David Anderson
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2004-2013 David Anderson
+ * Copyright (C) 2004-2013 Red Hat, Inc. All rights reserved.
  * Copyright (C) 2005  FUJITSU LIMITED
  * Copyright (C) 2005  NEC Corporation
  *
@@ -224,7 +224,7 @@ open_dump_file(char *file)
 
 	fd = open(file, O_RDONLY);
 	if (fd < 0) {
-		error(INFO, "diskdump / compressed kdump: unable to open dump file %s", file);
+		error(INFO, "diskdump / compressed kdump: unable to open dump file %s\n", file);
 		return FALSE;
 	}
 
@@ -297,6 +297,92 @@ x86_process_elf_notes(void *note_ptr, unsigned long size_note)
 		process_elf32_notes(note_ptr, size_note);
 }
 
+#if defined(__i386__) && defined(ARM)
+/*
+ * The kdump_sub_header member offsets are different when the crash 
+ * binary is built natively on an ARM host vs. when built with  
+ * "make target=ARM" on an x86/x86_64 host.  This is because the
+ * off_t structure members will be aligned on an 8-byte boundary when
+ * compiled as an ARM binary -- which will be reflected in the 
+ * kdump_sub_header in a compressed ARM kdump.  
+ *
+ * When crash is compiled as an x86 binary, these are the 
+ * structure's offsets:
+ * 
+ * struct kdump_sub_header {
+ * [0]     unsigned long   phys_base;
+ * [4]     int             dump_level;         /  header_version 1 and later  /
+ * [8]     int             split;              /  header_version 2 and later  /
+ * [12]    unsigned long   start_pfn;          /  header_version 2 and later  /
+ * [16]    unsigned long   end_pfn;            /  header_version 2 and later  /
+ * [20]    off_t           offset_vmcoreinfo;  /  header_version 3 and later  /
+ * [28]    unsigned long   size_vmcoreinfo;    /  header_version 3 and later  /
+ * [32]    off_t           offset_note;        /  header_version 4 and later  /
+ * [40]    unsigned long   size_note;          /  header_version 4 and later  /
+ * [44]    off_t           offset_eraseinfo;   /  header_version 5 and later  /
+ * [52]    unsigned long   size_eraseinfo;     /  header_version 5 and later  /
+ * };
+ * 
+ * But when compiled on an ARM processor, each 64-bit "off_t" would be pushed
+ * up to an 8-byte boundary:
+ * 
+ * struct kdump_sub_header {
+ * [0]     unsigned long   phys_base;
+ * [4]     int             dump_level;         /  header_version 1 and later  /
+ * [8]     int             split;              /  header_version 2 and later  /
+ * [12]    unsigned long   start_pfn;          /  header_version 2 and later  /
+ * [16]    unsigned long   end_pfn;            /  header_version 2 and later  /
+ * [24]    off_t           offset_vmcoreinfo;  /  header_version 3 and later  /
+ * [32]    unsigned long   size_vmcoreinfo;    /  header_version 3 and later  /
+ * [40]    off_t           offset_note;        /  header_version 4 and later  /
+ * [48]    unsigned long   size_note;          /  header_version 4 and later  /
+ * [56]    off_t           offset_eraseinfo;   /  header_version 5 and later  /
+ * [62]    unsigned long   size_eraseinfo;     /  header_version 5 and later  /
+ * };
+ * 
+ */
+
+struct kdump_sub_header_ARM_target {
+        unsigned long   phys_base;
+        int             dump_level;         /* header_version 1 and later */
+        int             split;              /* header_version 2 and later */
+        unsigned long   start_pfn;          /* header_version 2 and later */
+        unsigned long   end_pfn;            /* header_version 2 and later */
+	int		pad1;
+        off_t           offset_vmcoreinfo;  /* header_version 3 and later */
+        unsigned long   size_vmcoreinfo;    /* header_version 3 and later */
+	int 		pad2;	
+        off_t           offset_note;        /* header_version 4 and later */
+        unsigned long   size_note;          /* header_version 4 and later */
+	int 		pad3;	
+        off_t           offset_eraseinfo;   /* header_version 5 and later */
+        unsigned long   size_eraseinfo;     /* header_version 5 and later */
+};
+
+static void
+arm_kdump_header_adjust(int header_version)
+{
+	struct kdump_sub_header *kdsh;
+	struct kdump_sub_header_ARM_target *kdsh_ARM_target;
+
+	kdsh = dd->sub_header_kdump;
+	kdsh_ARM_target = (struct kdump_sub_header_ARM_target *)kdsh;
+
+	if (header_version >= 3) {
+		kdsh->offset_vmcoreinfo = kdsh_ARM_target->offset_vmcoreinfo; 
+		kdsh->size_vmcoreinfo = kdsh_ARM_target->size_vmcoreinfo;
+	}
+	if (header_version >= 4) {
+		kdsh->offset_note = kdsh_ARM_target->offset_note;
+		kdsh->size_note = kdsh_ARM_target->size_note;
+	}
+	if (header_version >= 5) {
+		kdsh->offset_eraseinfo = kdsh_ARM_target->offset_eraseinfo;
+		kdsh->size_eraseinfo = kdsh_ARM_target->size_eraseinfo;
+	}
+}
+#endif  /* __i386__ && ARM */
+
 static int 
 read_dump_header(char *file)
 {
@@ -397,8 +483,8 @@ restart:
                 error(WARNING, "%s: invalid nr_cpus value: %d\n",
                         DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
                         header->nr_cpus);
-		if (!machine_type("S390") && !machine_type("S390X")) {
-			/* s390 can get register information also from memory */
+		if (!machine_type("S390") && !machine_type("S390X") &&
+		    !machine_type("X86") && !machine_type("X86_64")) {
 			if (DISKDUMP_VALID())
 				goto err;
 		}
@@ -447,6 +533,10 @@ restart:
 			}
 		}
 		dd->sub_header_kdump = sub_header_kdump;
+
+#if defined(__i386__) && defined(ARM)
+		arm_kdump_header_adjust(header->header_version);
+#endif
 	}
 
 	/* read memory bitmap */
@@ -561,6 +651,11 @@ restart:
 		(sub_header_kdump->size_eraseinfo))
 		pc->flags2 |= ERASEINFO_DATA;
 
+	if (KDUMP_CMPRS_VALID() && (dd->header->header_version >= 3) &&
+		dd->sub_header_kdump->offset_vmcoreinfo &&
+		dd->sub_header_kdump->size_vmcoreinfo)
+		pc->flags2 |= VMCOREINFO;
+
 	/* For split dumpfile */
 	if (KDUMP_CMPRS_VALID()) {
 		is_split = ((dd->header->header_version >= 2) &&
@@ -686,6 +781,13 @@ is_diskdump(char *file)
 	dd->flags |= SNAPPY_SUPPORTED;
 #endif
 
+	if ((pc->flags2 & GET_LOG) && KDUMP_CMPRS_VALID()) {
+		pc->dfd = dd->dfd;
+		pc->readmem = read_diskdump;
+		pc->flags |= DISKDUMP;
+		get_log_from_vmcoreinfo(file, vmcoreinfo_read_string);
+	}
+
 	return TRUE;
 }
 
@@ -1485,14 +1587,26 @@ __diskdump_memory_dump(FILE *fp)
 			fprintf(fp, "%s\n", dump_level ? ")" : "");
 		} else
 			fprintf(fp, "(unknown)\n");
-		if (KDUMP_SPLIT()) {
-			fprintf(fp, "           start_pfn: %lu\n", dd->sub_header_kdump->start_pfn);
-			fprintf(fp, "             end_pfn: %lu\n", dd->sub_header_kdump->end_pfn);
+
+		if (dh->header_version >= 2) {
+			fprintf(fp, "               split: %d\n", kdsh->split);
+			fprintf(fp, "           start_pfn: ");
+			if (KDUMP_SPLIT())
+				fprintf(fp, "%ld (0x%lx)\n", 
+					kdsh->start_pfn, kdsh->start_pfn);
+			else
+				fprintf(fp, "(unused)\n");
+			fprintf(fp, "             end_pfn: ");
+			if (KDUMP_SPLIT())
+				fprintf(fp, "%ld (0x%lx)\n", 
+					kdsh->end_pfn, kdsh->end_pfn);
+			else
+				fprintf(fp, "(unused)\n");
 		}
 		if (dh->header_version >= 3) {
-			fprintf(fp, "   offset_vmcoreinfo: %lu (0x%lx)\n",
-				(ulong)dd->sub_header_kdump->offset_vmcoreinfo,
-				(ulong)dd->sub_header_kdump->offset_vmcoreinfo);
+			fprintf(fp, "   offset_vmcoreinfo: %llu (0x%llx)\n",
+				(ulonglong)dd->sub_header_kdump->offset_vmcoreinfo,
+				(ulonglong)dd->sub_header_kdump->offset_vmcoreinfo);
 			fprintf(fp, "     size_vmcoreinfo: %lu (0x%lx)\n",
 				dd->sub_header_kdump->size_vmcoreinfo,
 				dd->sub_header_kdump->size_vmcoreinfo);
@@ -1502,9 +1616,9 @@ __diskdump_memory_dump(FILE *fp)
 			}
 		}
 		if (dh->header_version >= 4) {
-			fprintf(fp, "         offset_note: %lu (0x%lx)\n",
-				(ulong)dd->sub_header_kdump->offset_note,
-				(ulong)dd->sub_header_kdump->offset_note);
+			fprintf(fp, "         offset_note: %llu (0x%llx)\n",
+				(ulonglong)dd->sub_header_kdump->offset_note,
+				(ulonglong)dd->sub_header_kdump->offset_note);
 			fprintf(fp, "           size_note: %lu (0x%lx)\n",
 				dd->sub_header_kdump->size_note,
 				dd->sub_header_kdump->size_note);
@@ -1519,9 +1633,9 @@ __diskdump_memory_dump(FILE *fp)
 			dump_nt_prstatus_offset(fp);
 		}
 		if (dh->header_version >= 5) {
-			fprintf(fp, "    offset_eraseinfo: %lu (0x%lx)\n",
-				(ulong)dd->sub_header_kdump->offset_eraseinfo,
-				(ulong)dd->sub_header_kdump->offset_eraseinfo);
+			fprintf(fp, "    offset_eraseinfo: %llu (0x%llx)\n",
+				(ulonglong)dd->sub_header_kdump->offset_eraseinfo,
+				(ulonglong)dd->sub_header_kdump->offset_eraseinfo);
 			fprintf(fp, "      size_eraseinfo: %lu (0x%lx)\n",
 				dd->sub_header_kdump->size_eraseinfo,
 				dd->sub_header_kdump->size_eraseinfo);
