@@ -1,8 +1,8 @@
 /* gdb_interface.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2013 David Anderson
+ * Copyright (C) 2002-2013 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -485,6 +485,8 @@ dump_gnu_request(struct gnu_request *req, int in_gdb)
                 console("%sGNU_FROM_TTY_OFF", others++ ? "|" : "");
         if (req->flags & GNU_NO_READMEM)
                 console("%sGNU_NO_READMEM", others++ ? "|" : "");
+        if (req->flags & GNU_VAR_LENGTH_TYPECODE)
+                console("%sGNU_VAR_LENGTH_TYPECODE", others++ ? "|" : "");
 	console(")\n");
 
         console("addr: %lx ", req->addr);
@@ -850,26 +852,29 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 		    text_value_cache_byte(addr, (unsigned char *)p1)) 
 			return TRUE;
 
-		if (readmem(addr, memtype, locbuf, SIZEOF_32BIT,
-                    "gdb_readmem_callback", RETURN_ON_ERROR)) {
-			*p1 = locbuf[0];
+		if (!readmem(addr, memtype, locbuf, SIZEOF_32BIT,
+		    "gdb_readmem_callback", RETURN_ON_ERROR)) 
+			return FALSE;
+
+		*p1 = locbuf[0];
+		if (memtype == KVADDR) {
 			p2 = (uint32_t *)locbuf;
 			text_value_cache(addr, *p2, 0);
-			return TRUE;
 		}
-		break;
+		return TRUE;
 
 	case SIZEOF_32BIT:
-                if ((memtype == KVADDR) && text_value_cache(addr, 0, buf)) 
+		if ((memtype == KVADDR) && text_value_cache(addr, 0, buf)) 
 			return TRUE;
 
-		if (readmem(addr, memtype, buf, SIZEOF_32BIT, 
-		    "gdb_readmem callback", FAULT_ON_ERROR)) {
-                       	text_value_cache(addr, 
+		if (!readmem(addr, memtype, buf, SIZEOF_32BIT, 
+		    "gdb_readmem callback", RETURN_ON_ERROR))
+			return FALSE;
+
+		if (memtype == KVADDR)
+			text_value_cache(addr, 
 				(uint32_t)*((uint32_t *)buf), NULL);
-			return TRUE;
-                }
-		break;
+		return TRUE;
 	}
 
 	return(readmem(addr, memtype, buf, len,
